# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['leprikon',
 'leprikon.admin',
 'leprikon.api',
 'leprikon.cms_plugins',
 'leprikon.conf',
 'leprikon.forms',
 'leprikon.forms.reports',
 'leprikon.management',
 'leprikon.management.commands',
 'leprikon.migrations',
 'leprikon.models',
 'leprikon.search_indexes',
 'leprikon.site',
 'leprikon.templatetags',
 'leprikon.views',
 'leprikon.views.reports']

package_data = \
{'': ['*'],
 'leprikon': ['locale/cs/LC_MESSAGES/*',
              'locale/en/LC_MESSAGES/*',
              'static/*',
              'static/admin/css/*',
              'static/admin/js/admin/*',
              'static/leprikon/css/*',
              'static/leprikon/fonts/*',
              'static/leprikon/img/*',
              'static/leprikon/js/*',
              'templates/admin/*',
              'templates/admin/inc/*',
              'templates/admin/leprikon/*',
              'templates/admin/leprikon/coursediscount/*',
              'templates/admin/leprikon/eventdiscount/*',
              'templates/admin/leprikon/message/*',
              'templates/admin/leprikon/messagerecipient/*',
              'templates/admin/leprikon/orderablediscount/*',
              'templates/admin/leprikon/subjectpayment/*',
              'templates/auth/widgets/*',
              'templates/cms/toolbar/items/*',
              'templates/leprikon/*',
              'templates/leprikon/admin/*',
              'templates/leprikon/cms/*',
              'templates/leprikon/cms/course/*',
              'templates/leprikon/cms/course_list/*',
              'templates/leprikon/cms/event/*',
              'templates/leprikon/cms/event_list/*',
              'templates/leprikon/cms/leader/*',
              'templates/leprikon/cms/leader_list/*',
              'templates/leprikon/cms/orderable/*',
              'templates/leprikon/cms/orderable_list/*',
              'templates/leprikon/payment_pdf/*',
              'templates/leprikon/payment_received/*',
              'templates/leprikon/registration_approved/*',
              'templates/leprikon/registration_canceled/*',
              'templates/leprikon/registration_payment_request/*',
              'templates/leprikon/registration_pdf/*',
              'templates/leprikon/registration_received/*',
              'templates/leprikon/registration_refused/*',
              'templates/leprikon/reports/*',
              'templates/leprikon/static/*',
              'templates/leprikon/subject_journal_pdf/*',
              'templates/leprikon/widgets/*',
              'templates/rocketchat/*']}

install_requires = \
['Django>=3.1.4,<4.0.0',
 'django-bankreader>=0.6.1,<0.7.0',
 'django-cms>=3.8.0,<4.0.0',
 'django-cron>=0.5.1,<0.6.0',
 'django-filer>=2.0.2,<3.0.0',
 'django-haystack>=3.0,<4.0',
 'django-localflavor>=3.0.1,<4.0.0',
 'django-mathfilters>=1.0.0,<2.0.0',
 'django-pays>=0.1.0,<0.2.0',
 'django-qr-code>=2.0.1,<3.0.0',
 'django-redis>=4.12.1,<5.0.0',
 'djangocms-link>=3.0.0,<4.0.0',
 'djangocms-snippet>=3.0.0,<4.0.0',
 'djangocms-style>=3.0.0,<4.0.0',
 'djangocms-text-ckeditor>=4.0.0,<5.0.0',
 'djangocms-video>=3.0.0,<4.0.0',
 'html2rml>=0.3.0,<0.4.0',
 'ipython>=7.19.0,<8.0.0',
 'pyexcel-xlsxw>=0.6.1,<0.7.0',
 'pymongo>=3.11.2,<4.0.0',
 'rocketchat-API>=1.12.1,<2.0.0',
 'sentry-sdk>=0.19.5,<0.20.0',
 'social-auth-app-django>=4.0.0,<5.0.0',
 'trml2pdf>=0.5.0,<0.6.0']

setup_kwargs = {
    'name': 'leprikon',
    'version': '3.0.0',
    'description': 'Django CMS based IS for education',
    'long_description': None,
    'author': 'Jakub Dorňák',
    'author_email': 'jakub.dornak@misli.cz',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
